//--------------------------------------------------------------------------------
// Project LibPhylo
//
// Copyright (C) 2000 Andreas Heger All rights reserved
//
// Author: Andreas Heger <heger@ebi.ac.uk>
//
// $Id: HelpersTree.cpp,v 1.2 2004/06/02 12:14:34 aheger Exp $
//--------------------------------------------------------------------------------    

#include <iostream>
#include <string>
#include <vector>

#ifdef WITH_DMALLOC
#include <dmalloc.h>
#endif

#include "Tree.h"
#include "HelpersTree.h"


using namespace std;

namespace PhyloLib {
  
void writeNewHampshire( std::ostream & output, const Tree * tree, const std::vector<std::string> * labels ) {
#ifdef DEBUG
    if (verbose > LL3) cout << "writeNewHampshire( std::ostream &, const Tree *, const std::vector<std::string> *)" << endl;
#endif

    TYPE_NODE node;
    std::vector<TYPE_NODE> node_stack;

    TYPE_NODE root = tree->getRoot();

    node_stack.push_back( root ); // push root on stack

    bool docomma = false;

    /* after hmmer-2.1 - cluster.c*/

    TYPE_NODE numleaves = tree->getNumLeaves();
    TYPE_NODE last_interior_node = 2 * numleaves - 1;
    TYPE_WEIGHT weight = 0;
    while (!node_stack.empty()) {

        node = node_stack.back();
        node_stack.pop_back();
#ifdef DEBUG
        // cout << "Processing node " << node << endl;
#endif

        if (node != root)
            weight = tree->getWeight( node, tree->getParent( node ));


        if (node < numleaves) {                   // process leaves: print name:branchlength
            if (docomma)
                output << ",";

            if (labels) {
                output << (*labels)[node];  // this is a bit awkward
            } else {
                output << node;
            }

            output << ":" << weight;
            docomma = true;

        } else {
            if (node <= last_interior_node) {             // process interior nodes
                if (docomma) output << "," << endl;

                // 1) print a '('
                output << "(";

                // 2) push on stack: ), right child, left child

                node_stack.push_back( node + numleaves );
                node_stack.push_back( tree->getRightChild(node) );
                node_stack.push_back( tree->getLeftChild(node) );

                // 3) record branch lengths
                docomma = false;

            } else {                                      // close interior node
                // print a ):branchlength (if root, just print ")" )
                if (node == root + numleaves) {
                    output << ")\n";
                } else {
                    node = node - numleaves;
                    output << "):" << tree->getWeight( node, tree->getParent(node) );
                }
 
                docomma = true;
            }
        }
    }
}
   

} // namespace PhyloLib
