//--------------------------------------------------------------------------------
// Project LibAlign
//
// Copyright (C) 2000 Andreas Heger All rights reserved
//
// Author: Andreas Heger <heger@ebi.ac.uk>
//
// $Id: ImplDistorDummy.cpp,v 1.1.1.1 2002/07/08 21:20:17 heger Exp $
//--------------------------------------------------------------------------------    


#include <iostream>
#include "ImplDistorDummy.h"
#include "Matrix.h"
#include "PhyloException.h"

using namespace std;

namespace PhyloLib {

//-------------------------> factory functions <-------------------------------------------------------------------------------
Distor * makeDistorDummy( const Matrix * matrix) {
  return new ImplDistorDummy( matrix);
}

//---------------------------------------------------------< constructors and destructors >--------------------------------------
ImplDistorDummy::ImplDistorDummy ( const Matrix * matrix) : ImplDistor(), mMatrix(matrix) {
}
		       
ImplDistorDummy::~ImplDistorDummy () {
#ifdef DEBUG
  if (verbose > LL3) cout << "ImplDistorDummy::~ImplDistorDummy() " << endl;
#endif
}

ImplDistorDummy::ImplDistorDummy (const ImplDistorDummy & src ) : ImplDistor( src ), mMatrix(src.mMatrix) {
}


//--------------------------------------------------------------------------------------------------------------------------------
TYPE_MATRIX_VALUE ImplDistorDummy::getMaximumPossibleDistance() const {
    return mMatrix->getMaximum();
}

//--------------------------------------------------------------------------------------------------------------------------------
Matrix * ImplDistorDummy::calculateMatrix( Matrix * matrix, const AlignLib::MultipleAlignment * multali) const {

    TYPE_MATRIX_SIZE i, j;
    TYPE_MATRIX_SIZE width = mMatrix->getWidth();
    
    matrix->setWidth( width );

    // This is definitely not efficient
    for (i = 0; i < width - 1; i++) 
	for (j = i + 1; j < width; j++) 
	    (*matrix)(i, j) = (*mMatrix)(i, j);
    
    return matrix;
} 
    
//--------------------------------------------------------------------------------------------------------------------------------
TYPE_MATRIX_VALUE ImplDistorDummy::calculateDistance( const std::string & s_row_1, const std::string & s_row_2) const {
#ifdef DEBUG
  if (verbose > LL3) cout << "TYPE_MATRIX_VALUE ImplDistorDummy::calculateDistance( TYPE_MATRIX_SIZE, TYPE_MATRIX_SIZE) const" << endl;
#endif
  
  return 0;
}


} // namespace PhyloLib
