//--------------------------------------------------------------------------------
// Project LibAlign
//
// Copyright (C) 2000 Andreas Heger All rights reserved
//
// Author: Andreas Heger <heger@ebi.ac.uk>
//
// $Id: ImplDistorKimura.cpp,v 1.1.1.1 2002/07/08 21:20:17 heger Exp $
//--------------------------------------------------------------------------------    


#include <iostream>
#include <iomanip>
#include <string>
#include <math.h>

#include <alignlib/Translator.h>
#include <alignlib/HelpersTranslator.h>
#include <alignlib/MultipleAlignment.h>
#include "ImplDistorKimura.h"

using namespace std;

namespace PhyloLib {

#define MAX_DIFFERENCE 0.85
#define MAX_DISTANCE 5.2030

//-------------------------> factory functions <-------------------------------------------------------------------------------
Distor * makeDistorKimura() {
  return new ImplDistorKimura();
}

//---------------------------------------------------------< constructors and destructors >--------------------------------------
ImplDistorKimura::ImplDistorKimura () : ImplDistor() {
}
		       
ImplDistorKimura::~ImplDistorKimura () {
#ifdef DEBUG
  if (verbose > LL3) cout << "ImplDistorKimura::~ImplDistorKimura()" << endl;
#endif
}

ImplDistorKimura::ImplDistorKimura (const ImplDistorKimura & src ) : ImplDistor( src ) {
}


//--------------------------------------------------------------------------------------------------------------------------------
TYPE_MATRIX_VALUE ImplDistorKimura::getMaximumPossibleDistance() const {
    return MAX_DISTANCE;
}

//--------------------------------------------------------------------------------------------------------------------------------
TYPE_MATRIX_VALUE ImplDistorKimura::calculateDistance(const std::string & s_row_1, const std::string & s_row_2) const {

  unsigned int i;
  unsigned int identities = 0;		// number of identities
  unsigned int n_nongaps = 0;		// normalize over non-gap-positions
  
  unsigned char gap_char = AlignLib::getDefaultTranslator()->getGapChar();
  
  for (i = 0; i < s_row_1.length(); i++) {
    if ((s_row_1[i] != gap_char) && (s_row_2[i] != gap_char)) {
      n_nongaps ++;
      if (s_row_1[i] == s_row_2[i]) 
	identities++;
    }
  }

#ifdef DEBUG
  cout << "Comparison between " << s_row_1 << " and " << s_row_2 << ": non_gaps=" << n_nongaps << " identitities=" << identities << endl;
#endif

  double pdiff;
  if (n_nongaps > 0)
    pdiff = 1.0 - (double)identities / (double)n_nongaps;
  else
    pdiff = 1.0;
  
  if (pdiff > MAX_DIFFERENCE) 
    return MAX_DISTANCE;
  else
    return -log( 1.0 - pdiff - 0.2 * pdiff * pdiff);
}



//--------------------------------------------------------------------------------------------------------------------------------

} // namespace AlignLib
