//--------------------------------------------------------------------------------
// Project LibAlign
//
// Copyright (C) 2000 Andreas Heger All rights reserved
//
// Author: Andreas Heger <heger@ebi.ac.uk>
//
// $Id: ImplMatrix.h,v 1.1.1.1 2002/07/08 21:20:17 heger Exp $
//--------------------------------------------------------------------------------

#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef IMPL_MATRIX_H
#define IMPL_MATRIX_H 1

#include <iosfwd>

#include "Matrix.h"

namespace PhyloLib {

/**

   -> memory is allocated when the matrix is created and not given free, until it
   is deleted (i.e. no partial memory freeing during shrinks)

   @author Andreas Heger
   @version $Id: ImplMatrix.h,v 1.1.1.1 2002/07/08 21:20:17 heger Exp $
   @short contains a matrix
*/ 

  // i.e., for invalid requests return first element. This is bad, but
  // did not want to do all the checking
#define NO_INDEX 0		

class ImplMatrix : public Matrix {
 public:
  
  /* constructors and desctructors------------------------------------------------------- */

  /** empty constructor */
  ImplMatrix ();

  /** create matrix of width and set all values to default_value */
  ImplMatrix( TYPE_MATRIX_SIZE width, TYPE_MATRIX_VALUE default_value);

  /** load array */
  ImplMatrix( TYPE_MATRIX_SIZE width, TYPE_MATRIX_VALUE * source);

  /** copy constructor */
  ImplMatrix (const ImplMatrix &);

  /** destructor */
  virtual ~ImplMatrix ();

  /* member access functions--------------------------------------------------------------- */
  /** return width of matrix */
  virtual TYPE_MATRIX_SIZE getWidth() const;

  /** sets the width of matrix, old matrix is deleted */
  virtual void setWidth(TYPE_MATRIX_SIZE width);

  /** return size (rows * columns) of matrix */
  virtual TYPE_MATRIX_SIZE getSize() const;

  /** return the minimum value of the matrix */
  virtual TYPE_MATRIX_VALUE getMinimum() const;

  /** return the minimum value of the matrix + coordinates */
  virtual TYPE_MATRIX_VALUE getMinimum( COORD & coordinates) const;

  /** return the maximum value of the matrix */
  virtual TYPE_MATRIX_VALUE getMaximum() const;

  /** return the maximum value of the matrix */
  virtual TYPE_MATRIX_VALUE getMaximum( COORD & coordinates) const;
  
  /** return element */
  virtual TYPE_MATRIX_VALUE operator()(TYPE_MATRIX_SIZE row, TYPE_MATRIX_SIZE col) const;
  virtual TYPE_MATRIX_VALUE getElement(TYPE_MATRIX_SIZE row, TYPE_MATRIX_SIZE col) const;

  /** set element */
  virtual TYPE_MATRIX_VALUE & operator()(TYPE_MATRIX_SIZE row, TYPE_MATRIX_SIZE col);
  virtual void setElement(TYPE_MATRIX_SIZE row, TYPE_MATRIX_SIZE col, TYPE_MATRIX_VALUE value);

  /** delete last row/column from matrix */
  virtual void Shrink();

  /** swap two columns/rows */
  virtual void Swap( TYPE_MATRIX_SIZE col_1, TYPE_MATRIX_SIZE col_2 );

  /** read information from stream */
  virtual void Read ( std::istream & input ) const;

  virtual void Write( std::ostream & output ) const;


 protected:
  
  /** get row for given index */
  virtual TYPE_MATRIX_SIZE getRow( TYPE_MATRIX_SIZE index ) const;
 
  /** get column for given index */
  virtual TYPE_MATRIX_SIZE getColumn( TYPE_MATRIX_SIZE index ) const;
 
  /** return mapped index for row and column */
  virtual TYPE_MATRIX_SIZE getIndex(TYPE_MATRIX_SIZE row, TYPE_MATRIX_SIZE col) const;   

  /** allocate memory */
  virtual void allocateMemory();

  /** allocate memory */
  virtual void freeMemory();

  /** calculate the memory size of the matrix given its width*/
  virtual void calculateSize();

  /** the width of the matrix */
  TYPE_MATRIX_SIZE mWidth;
 
  /** the size of the matrix in terms of elements of type TYPE_MATRIX*/
  TYPE_MATRIX_SIZE mSize;

  /** a pointer to the matrix */
  TYPE_MATRIX_VALUE * mMatrix;

};


}

#endif /* _ALIGNATUMSEQUENCE_H */

