//--------------------------------------------------------------------------------
// Project LibAlign
//
// Copyright (C) 2000 Andreas Heger All rights reserved
//
// Author: Andreas Heger <heger@ebi.ac.uk>
//
// $Id: Matrix.h,v 1.1.1.1 2002/07/08 21:20:17 heger Exp $
//--------------------------------------------------------------------------------

#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef MATRIX_H
#define MATRIX_H 1

#include <iostream>
#include "phylolib.h"

namespace PhyloLib {

/**
   Matrix object, base class for distance and similarity matrices. 
   
   simple and painful
   do not use for heavy computation as the accession operator is
   overloaded. Instead, use matrix template library.
   
   first index is row, second index is column

   @author Andreas Heger
   @version $Id: Matrix.h,v 1.1.1.1 2002/07/08 21:20:17 heger Exp $
   @short contains a matrix
*/ 

class Matrix {
  /* friends---------------------------------------------------------------------------- */
  friend std::ostream & operator<<( std::ostream &, const Matrix &);
  friend std::istream & operator>>( std::istream &, Matrix &);
  /* class member functions-------------------------------------------------------------- */
 public:
  
  /* constructors and desctructors------------------------------------------------------- */

  /** empty constructor */
  Matrix ();

  /** copy constructor */
  Matrix (const Matrix &);

  /** destructor */
  virtual ~Matrix ();

  /* member access functions--------------------------------------------------------------- */
  /** return width of matrix */
  virtual TYPE_MATRIX_SIZE getWidth() const = 0;

  /** sets the width of matrix, old matrix is deleted */
  virtual void setWidth(TYPE_MATRIX_SIZE width) = 0;

  /** return size (rows * columns) of matrix */
  virtual TYPE_MATRIX_SIZE getSize() const = 0;

  /** return the minimum value of the matrix */
  virtual TYPE_MATRIX_VALUE getMinimum() const = 0;

  /** return the minimum value of the matrix + coordinates */
  virtual TYPE_MATRIX_VALUE getMinimum( COORD & coordinates) const = 0;

  /** return the maximum value of the matrix */
  virtual TYPE_MATRIX_VALUE getMaximum() const = 0;

  /** return the maximum value of the matrix */
  virtual TYPE_MATRIX_VALUE getMaximum( COORD & coordinates) const = 0;
  
  /** return element */
  virtual TYPE_MATRIX_VALUE operator()(TYPE_MATRIX_SIZE row, TYPE_MATRIX_SIZE col) const = 0;
  virtual TYPE_MATRIX_VALUE getElement(TYPE_MATRIX_SIZE row, TYPE_MATRIX_SIZE col) const = 0;

  /** set element */
  virtual TYPE_MATRIX_VALUE & operator()(TYPE_MATRIX_SIZE row, TYPE_MATRIX_SIZE col) = 0;
  virtual void setElement(TYPE_MATRIX_SIZE row, TYPE_MATRIX_SIZE col, TYPE_MATRIX_VALUE value) = 0;

  /** delete last row/column from matrix */
  virtual void Shrink() = 0;

  /** swap two columns/rows */
  virtual void Swap( TYPE_MATRIX_SIZE col_1, TYPE_MATRIX_SIZE col_2 ) = 0;

  /** read information from stream */
  virtual void Read ( std::istream & input ) const = 0;

  virtual void Write( std::ostream & output ) const = 0;

};

}

#endif /* _ALIGNATUMSEQUENCE_H */

